package com.qizhen.healsphere.repository.neo4j;

import com.qizhen.healsphere.config.Neo4jUtil;
import org.neo4j.driver.v1.*;
import org.neo4j.driver.v1.types.Node;
import org.springframework.stereotype.Repository;

import java.util.*;
import java.util.stream.Collectors;

@Repository
public class BaseRelationshipRepository {

    public void createRelationship(String startLabel, Long startId, String endLabel, Long endId, String relationshipType, Map<String, Object> properties) {
        StringBuilder setClause = new StringBuilder();
        properties.forEach((key, value) -> setClause.append("r.").append(key).append(" = '").append(value).append("'").append(", "));

        if (setClause.length() > 0) {
            setClause.setLength(setClause.length() - 2);
        }

        String query = String.format(
                "MATCH (a:`%s`), (b:`%s`) WHERE id(a) = $startId AND id(b) = $endId " +
                        "MERGE (a)-[r:`%s`]->(b) " +
                        "SET %s RETURN r",
                startLabel, endLabel, relationshipType, setClause
        );

        try(Session session = Neo4jUtil.getSession()) {
            session.run(query, Values.parameters("startId", startId,"endId",endId));
        }
    }

    public void updateRelationship(String oldRelationshipType, String newRelationshipType) {

        String query = String.format(
                "MATCH (a)-[r:`%s`]->(b) " +
                        "CREATE (a)-[new_r:`%s`]->(b) " +
                        "SET new_r += r " +
                        "DELETE r",
                oldRelationshipType, newRelationshipType
        );

        try(Session session = Neo4jUtil.getSession()) {
            session.run(query);
        }
    }

    public List<String> findRelationshipNames(String name) {

        String query = "match ()-[r]-() where type(r) contains $name return distinct type(r)";

        try(Session session = Neo4jUtil.getSession()) {
            StatementResult result = session.run(query, Values.parameters("name", name));
            List<String> list = new ArrayList<>();
            if(result.hasNext()) {
                List<Record> records = result.list();
                for (Record record : records) {
                    list.addAll(record.values().stream().map(Value::asString).collect(Collectors.toList()));
                }
            }
            return list;
        }
    }



    public void deleteRelationship(String startLabel, Long startId, String endLabel, Long endId, String relationshipType) {
        String query = String.format(
                "MATCH (a:`%s`)-[r:`%s`]->(b:`%s`) WHERE id(a) = $startId AND id(b) = $endId DELETE r",
                startLabel, relationshipType, endLabel
        );

        try(Session session = Neo4jUtil.getSession()) {
            session.run(query, Values.parameters("startId", startId,"endId",endId));
        }
    }
}